@@ -4,6 +4,8 @@ use 5.006;
 use strict;
 use warnings;
 
+use File::Find;
+
 require Exporter;
 
 our @ISA = qw(Exporter);
@@ -17,20 +19,43 @@ our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw(
 	
 );
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 
 sub random_file {
-	my ($dir, $check) = _params_random_file(@_);
+	my @params = my ($dir, $check, $recursive) = _params_random_file(@_);
 	
+	return $recursive ? _random_file_recursive    (@params)
+	                  : _random_file_non_recursive(@params);
+}
+
+sub _random_file_non_recursive {
+	my ($dir, $check) = @_;
+
 	opendir DIR, $dir or die "Can't open directory '$dir'";
 	my @files = grep {-f "$dir/$_" and _valid_file($check, $_)} (readdir DIR);
 	closedir DIR;
-	
+
 	return undef unless @files;
 	return $files[rand @files];
 }
 
+sub _random_file_recursive {
+	my ($dir, $check) = @_;
+
+	my $i = 1;
+	my $fname;
+
+	my $accept_routine = sub {
+		return unless -f; 
+		return unless _valid_file($check,$_);
+		$fname = $_ if rand($i++) < 1;	# Algorithm from Cookbook, chapter 8.6
+	};
+	find($accept_routine, $dir);
+	
+	return $fname;
+}
+
 sub _valid_file {
 	my ($check, $name) = @_; 
 	for (ref($check)) {
@@ -49,6 +74,7 @@ sub _params_random_file {
 	
 	my $dir   = $args{-dir}   || '.';    
 	my $check = $args{-check} || sub {"always O.K."};
+	my $recursive = $args{-recursive};   # defaults to undef, already default
 
 	$dir =~ s:[/\\]+$::;                 # /home/xyz != /home/xyz/
 	
@@ -57,7 +83,7 @@ sub _params_random_file {
 		    "not a '" . ref($check) . "'";
 	}
 		
-	return ($dir, $check);
+	return ($dir, $check, $recursive);
 }
 
 1;
@@ -75,8 +101,9 @@ File::Random - Perl module for random selecting of a file
 
   my $fname2 = random_file(-dir => $dir);
   
-  my $random_gif = random_file(-dir   => $dir,
-                               -check => qr/\.gif$/);
+  my $random_gif = random_file(-dir       => $dir,
+                               -check     => qr/\.gif$/,
+							   -recursive => 1);
 							   
   my $no_exe     = random_file(-dir   => $dir,
                                -check => sub {! -x});
@@ -98,20 +125,32 @@ or
   closedir DIR;
   my $randf = $files[rand @files];
  
+It also becomes very boring and very dangerous to write 
+randomly selection for subdirectory searching with special check-routines.
   
-=head2 FUNCTIONS
+=head2 FUNCTION random_file
+
+Returns a randomly selected file(name) from the specified directory
+If the directory is empty, undef will be returned. There 3 options:
+
+  my $file = random_file(
+     -dir         => $dir, 
+	 -check       => qr/.../, # or sub { .... }
+	 -recursive   => 1        # or 0
+  );
 
 =over
 
-=item random_file(-dir => $dir, -check => $sub_or_re)
+=item -dir
 
-Returns a randomly selected file(name) from the specified directory
-If the directory is empty, undef will be returned.
+Specifies the directory where file has to come from.
 
 Is the -dir option missing,
 a random file from the current directory will be used.
 That means '.' is the default for the -dir option.
 
+=item -check
+
 With the -check option you can either define
 a regex every filename has to follow,
 or a sub routine which gets the filename as argument.
@@ -121,6 +160,22 @@ than a regexp or a subroutine.
 A string like '/.../' won't work.
 I still work on that.
 
+The default is no checking (undef).
+
+=item -recursive
+
+Enables, that subdirectories are scanned for files, too.
+Every file, independent from its position in the file tree,
+has the same chance to be choosen.
+
+Every true value sets recursive behaviour on,
+every false value switches off.
+The default if false (undef).
+
+Note, that I programmed the recursive routine very defendly
+(using File::Find).
+So switch recursive on slowers the program a bit :-)
+
 =back
 
 =head2 EXPORT
@@ -133,35 +188,48 @@ C<use File::Random qw/:all/;>.
 
 =head1 TODO
 
-I think, I'll need some more options.
+I think, I'll need to expand the options.
 Instead of only one directory,
 it should be possible to take a random file from some directories.
-Even a recursive "search" should be included.
 
 The -check option doesn't except a string looking like regexp.
 In future versions there should be the possibility of passing a string
 like '/..../' instead of the regexp qr/.../';
 
+To create some aliases for the params is a good idea, too.
+I thought to make -d == -dir, -r == -recursive and -c == -check.
+(Only a lazy programmer is a good programmer).
+
 So I want to make it possible to write:
   
-  my $fname = random_file( -dir => '...', -recursive => 1, -check     => qr/\.html/ );
+  my $fname = random_file( -dir => '...', -recursive => 1, -check     => '/\.html/' );
 
 or even:
 
-  my $fname = random_file( -dir => [$dir1, $dir2, $dir3, ...], -r => 1, -check => sub {-M < 7} );
+  my $fname = random_file( -d => [$dir1, $dir2, $dir3, ...], -r => 1, -c => sub {-M < 7} );
 
 I also want to add a method C<content_of_random_file> and C<random_line>.
 
-The next thing, I'll implement is the -recursive option.
+The next thing, I'll implement is the content_of_random_file method.
 
 Just have a look some hours later.
 
+=head1 BUGS
+
+Oh, I hope none. I still have more test lines than function code.
+
+Well, but because I want some random data, it's a little bit hard to test.
+So a test could be wrong, allthough everything is O.K..
+To avoid it, I let many tests run,
+so that the chances for misproofing should be < 0.0000000001% or so.
+Even it has the disadvantage that the tests need really long :-(
+
 =head1 COPYRIGHT
 
 This Program is free software.
 You can change or redistribute it under the same condition as Perl itself.
 
-Copyright (c) 2002, Janek Schleicher
+Copyright (c) 2002, Janek Schleicher, E<lt>bigj@kamelfreund.deE<gt>
 
 =head1 AUTHOR
 
